from vvecon.rest_api.utils import ReqParser, Parser
from vvecon.rest_api.utils.Types import str__, int__


# -- User Parser -- #
class UserParser(Parser):
    """
    This class is to define requirement parses and simply the process
    """
